  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
  ;; Emacs-AIS Integration by Frank Piron, 2010
  ;; 
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; 
  (require 'compile)
  (require 'comint)
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Customize Environment
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;Set Path to AIS-Ini-File-here
    (setq ais-ini-file "c:\\ais\\bin\\ais.txt")
    ;Set Path to AIS-Client here
    (setq ais-client-exe "c:\\ais\\bin\\ais_client.exe")
    ;Set AIS-Output-Delimiter as defined in the format-line of the ais-ini-file
    (setq delimiter "|")
    ;; Eventually extend the compilation search path for compilation buffers
    (setq compilation-search-path (cons compilation-search-path
					  '(
					    ;; add more here
					    "c:\\ais\\lib"
                                            
					    )
					  )
	  )
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Customize Key-Bindings
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
    ;; Bind ais-reference to F3-Key
    (define-key global-map [f3] 'ais-reference)  
    ;; Bind ais-location to F4-Key
    (define-key global-map [f4] 'ais-location)  
    ;; Bind ais-unused to F5-Key
    (define-key global-map [f5] 'ais-unused) 
    ;; Bind ais-syntax to F6-Key
    (define-key global-map [f6] 'ais-syntax)  
    ;; Bind ais-close to F7-Key
    (define-key global-map [f7] 'ais-close)  
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; Functions
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;;
(defun current-line ()
  "Returns the current line number (in the buffer) of point."
  (interactive)
  ;Save narrowing State
  (save-restriction
    (widen)
    ;Save Point and Mark
    (save-excursion
      (beginning-of-line)
               (1+ (count-lines 1 (point))))))
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; AIS Reference
(defun ais-reference ()
  "Request the definition of the symbol around point"
  (interactive)
  (let (cmd filename ais-client-output 
            ref-filename found-line found-col)
    ;build shell command
    (setq cmd  (concat ais-client-exe 
		       " " 	
	       ais-ini-file 
		       " Reference " 
		       (buffer-file-name) 
		       " " 
		       (number-to-string (current-line))
		       " " 
		       (number-to-string (+ 1 (current-column)))
		       )
	  )
    ;and execute
    (setq ais-client-output (shell-command-to-string cmd))
    ;eventually cut off a leading ">" sign
    (if (string= (substring ais-client-output 0 1) ">")
	(setq ais-client-output (substring ais-client-output 2 nil))
    )
    ;split ais-client-output according to delimiter and extract filename or message
    (setq offset 1)
    ;delimiter ":" must be handled because of c:\\... like filenames
    (if (and (string= delimiter ":") 
	     (string= (substring ais-client-output 1 2) ":")
	     )
      (setq ref-filename  (concat (nth 0 (split-string ais-client-output delimiter))
                                  ":"
                                  (nth 1 (split-string ais-client-output delimiter))
                          ))
      (setq ref-filename  (nth 0 (split-string ais-client-output delimiter)))
      (setq offset 0)
    )
    ;if first portion of output is not a readable file
    (if (not (file-readable-p ref-filename))
      ;report error
      (message (concat "Error in reading File: " ref-filename))
      ;(message ais-client-output)
      ;else open source file
      (find-file-other-window ref-filename)
      ;get line and column of referred symbol
      (setq found-line (string-to-number (nth (+ 1 offset) (split-string ais-client-output delimiter))))
      (setq found-col  (string-to-number (nth (+ 2 offset) (split-string ais-client-output delimiter))))
      ;and move to the symbol
      (goto-line found-line)
      (forward-char (- found-col 1))
    )         
))
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; AIS Location
(defun ais-location ()
  "Request a list of all the locations where the symbol around point has been referenced"
  (interactive)
  (let (cmd filename line column ais-client-output 
            ref-file-list)
    ;build shell command
    (setq cmd  (concat ais-client-exe 
		       " " 
		       ais-ini-file 
		       " Location " 
		       (buffer-file-name) 
		       " " 
		       (number-to-string (current-line))
		       " " 
		       (number-to-string (+ 1 (current-column)))
		       )
	  )
    ;and execute
(progn
    (compile-internal cmd "No more references" "ais-location")
    ;wait for the process to finish
    (while (string= "run" (symbol-name (process-status "ais-location")))
      (sleep-for 0.1)
      )
    ;switch to the output buffer
    (switch-to-buffer-other-window "*ais-location*")
    ;remove "> " and replace delimiter by ":" to get the
    ;compilation-buffer behavior
    (goto-char (point-min))
    (while (search-forward "> " nil t) (replace-match "" nil t))
    (goto-char (point-min))
    (while (search-forward delimiter nil t) (replace-match ":" nil t))
    (goto-char (point-min))
    )
))
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; AIS Unused
(defun ais-unused ()
"Show all the declarations made in the buffer-file and all its dependencies that are unused. Performing this action on a main package will effectively show all the unused declarations for a specific program"
  (interactive)
  (let (cmd filename line column ais-client-output 
            ref-file-list)
    ;build shell command
    (setq cmd  (concat ais-client-exe 
		       " " 
		       ais-ini-file 
		       " Unused " 
		       (buffer-file-name) 
		       )
	  )
    ;and execute
(progn
    (message "ais-unused running. Please wait ...")
    (compile-internal cmd "No more references" "ais-unused")
    ;wait for the process to finish
    (while (string= "run" (symbol-name (process-status "ais-unused")))
      (sleep-for 0.1)
      )
    ;switch to the output buffer
    (switch-to-buffer-other-window "*ais-unused*")
    ;remove "> " and replace delimiter by ":" to get the
    ;compilation-buffer behavior
    (goto-char (point-min))
    (while (search-forward "> " nil t) (replace-match "" nil t))
    (goto-char (point-min))
    (while (search-forward delimiter nil t) (replace-match ":" nil t))
    (goto-char (point-min))
    )
))
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  ;; AIS Syntax
(defun ais-syntax ()
"Check the syntax of the file in buffer"
  (interactive)
  (let (cmd filename line column ais-client-output 
            ref-file-list)
    ;build shell command
    (setq cmd  (concat ais-client-exe 
		       " " 
		       ais-ini-file 
		       " Syntax " 
		       (buffer-file-name) 
		       )
	  )
    ;and execute
(progn
    (message "ais-syntax running. Please wait ...")
    (compile-internal cmd "No more references" "ais-syntax")
    ;wait for the process to finish
    (while (string= "run" (symbol-name (process-status "ais-syntax")))
      (sleep-for 0.1)
      )
    ;switch to the output buffer
    (switch-to-buffer-other-window "*ais-syntax*")
    ;remove "> " and replace delimiter by ":" to get the
    ;compilation-buffer behavior
    (goto-char (point-min))
    (while (search-forward "<Syntax Error> " nil t) (replace-match "" nil t))
    (goto-char (point-min))
    (while (search-forward delimiter nil t) (replace-match ":" nil t))
    (goto-char (point-min))
    )
))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; AIS Close
(defun ais-close ()
  "Shutdown Ada Information Server"
  (interactive)
  (let (cmd  ais-client-output)
    ;build shell command
    (setq cmd  (concat ais-client-exe 
		       " " 	
		       ais-ini-file 
		       " Close" 
		       )
	  )
    ;and execute
    (setq ais-client-output (shell-command-to-string cmd))
    (message "Ada Information Server shut down.")
    )
)
